import logging
import typing

from int_hash_lib import make_hs

logger = logging.Logger(__name__)


def build_ua_agg0(raw_data: str, salt: str) -> typing.Dict:
    result = {'raw_data': raw_data, 'data': None, 'hs': None, 'error': None}
    try:
        data = raw_data.split('[')[0]
        hs = make_hs(data, salt)
        result.update({'raw_data': raw_data, 'data': data, 'hs': hs, 'error': None})
    except (AttributeError, ValueError, TypeError, IndexError) as e:
        error = 'build_ua_agg0 error for value: %s, error: %s' % (raw_data, e)
        logger.error(error)
        result.update({'error': error})
    return result


def collect_user_agent_aggregates(raw_data: str, salt: str, **kwargs) -> typing.Dict:
    agg0 = build_ua_agg0(raw_data, salt)
    return {
        'agg0': agg0['hs'],
    }


__all__ = ['collect_user_agent_aggregates']



import logging
import typing

logger = logging.Logger(__name__)


def basic_normalize(data: str) -> typing.Optional[str]:
    try:
        return data.lower().strip() or None
    except AttributeError:
        logger.error('invalid data in "to_lower" function "%s"' % data)
    return data


def remove_points(data: str) -> typing.Optional[str]:
    try:
        return data.replace('.', '') or None
    except AttributeError:
        logger.error('invalid data in "remove_points" function "%s"' % data)
    return data


def non_digit(data: str) -> typing.Optional[str]:
    try:
        return ''.join(i for i in data if not i.isdigit()) or None
    except AttributeError:
        logger.error('invalid data in "non_digit_before_dog" function "%s"' % data)
    return data


def only_digit(data: str) -> typing.Optional[str]:
    try:
        return ''.join(i for i in data if i.isdigit()) or None
    except AttributeError:
        logger.error('invalid data in "only_digit_before_dog" function "%s"' % data)
    return data


def str_unique_sorted_symbols(data: str) -> typing.Optional[str]:
    try:
        return ''.join(i for i in sorted(set(data))) or None
    except AttributeError:
        logger.error('invalid data in "str_to_sorted_set" function "%s"' % data)
    return data



from .default import collect_hash_aggregates
from .email import collects_email_aggregates
from .ip_address import collect_ip_address_aggregates
from .phone import collect_phone_aggregates
from .user_agent import collect_user_agent_aggregates


class HashFunctions:
    COLLECT_HASH_AGGREGATES = 'collect_hash_aggregates'
    COLLECTS_EMAIL_AGGREGATES = 'collects_email_aggregates'
    COLLECT_IP_ADDRESS_AGGREGATES = 'collect_ip_address_aggregates'
    COLLECT_PHONE_AGGREGATES = 'collect_phone_aggregates'
    COLLECT_USER_AGENT_AGGREGATES = 'collect_user_agent_aggregates'



import logging
import typing

from int_hash_lib import make_hs
from int_hash_lib.utils.normalizer import basic_normalize, remove_points, only_digit, str_unique_sorted_symbols, non_digit

logger = logging.getLogger(__name__)


def is_gmail(raw_data: str) -> bool:
    if not raw_data:
        return False
    elif 'gmail' in raw_data:
        return True
    return False


def email_valid_list(raw_data: str) -> typing.Tuple:
    if not raw_data:
        return tuple()
    left, right = list(filter(lambda x: x, raw_data.split('@')))
    return tuple((left, right))


def build_email_agg0(raw_data: str, salt) -> typing.Dict:
    result = {'raw_data': raw_data, 'data': None, 'hs': None, 'error': None}
    try:
        data = basic_normalize(raw_data)
        left, right = email_valid_list(data)
        if is_gmail(right):
            left = remove_points(left)
        data = '@'.join((left, right))
        result.update({'raw_data': raw_data, 'data': data, 'hs': make_hs(data, salt), 'error': None})
    except (ValueError, TypeError) as e:
        error = 'build_email_agg0 error for value: %s, error: %s' % (raw_data, e)
        logger.error(error)
        result.update({'error': error})
    return result


def build_email_agg1(raw_data: str, salt) -> typing.Dict:
    result = {'raw_data': raw_data, 'data': None, 'hs': None, 'error': None}
    try:
        left, right = email_valid_list(raw_data)
        left = non_digit(left)
        hs = None if left is None else make_hs(left, salt)
        result.update({'raw_data': raw_data, 'data': left, 'hs': hs, 'error': None})
    except (AttributeError, ValueError, TypeError) as e:
        error = 'build_email_agg1 error for value: %s, error: %s' % (raw_data, e)
        logger.error(error)
        result.update({'error': error})
    return result


def build_email_agg2(raw_data: str, salt) -> typing.Dict:
    result = {'raw_data': raw_data, 'data': None, 'hs': None, 'error': None}
    try:
        left, right = list(filter(lambda x: x, raw_data.split('@')))
        left = only_digit(left)
        hs = None if left is None else make_hs(left, salt)
        result.update({'raw_data': raw_data, 'data': left, 'hs': hs, 'error': None})
    except (AttributeError, ValueError, TypeError) as e:
        error = 'build_email_agg1 error for value: %s, error: %s' % (raw_data, e)
        logger.error(error)
        result.update({'error': error})
    return result


def build_email_agg3(raw_data: str, salt) -> typing.Dict:
    result = {'raw_data': raw_data, 'data': None, 'hs': None, 'error': None}
    try:
        data = str_unique_sorted_symbols(raw_data)
        hs = None if data is None else make_hs(data, salt)
        result.update({'raw_data': raw_data, 'data': data, 'hs': hs, 'error': None})
    except (AttributeError, ValueError, TypeError) as e:
        error = 'build_email_agg1 error for value: %s, error: %s' % (raw_data, e)
        logger.error(error)
        result.update({'error': error})
    return result


def collects_email_aggregates(raw_data: str, salt: str, **kwargs):
    agg0 = build_email_agg0(raw_data, salt)
    agg1 = build_email_agg1(agg0['data'], salt)
    agg2 = build_email_agg2(agg0['data'], salt)
    agg3 = build_email_agg3(agg1['data'], salt)
    return {
        'agg0': agg0['hs'],
        'agg1': agg1['hs'],
        'agg2': agg2['hs'],
        'agg3': agg3['hs'],
    }


__all__ = ['collects_email_aggregates']



import csv
import datetime
import hashlib
import importlib
import logging
import typing
from io import StringIO

logger = logging.getLogger(__name__)


def str_or_None(v: str):
    if v is None or str(v).strip() == '':
        return
    return str(v)


def make_hs(s: str, salt: str) -> int:
    try:
        if s is None:
            return
        separator = b':'
        bs = bytes(s, encoding='utf-8')
        b_salt = bytes(salt, encoding='utf-8')
        data = bs + separator + b_salt
        hs = int(hashlib.sha256(data).hexdigest(), 32)
        return int(str(hs)[10:28])
    except Exception as e:
        logger.error('make_hash error : %s' % e)


def prepare_hash_by_key(k: str, v: typing.Any, salt: str, region: str, mapping: typing.Dict) -> typing.Dict:
    mod = importlib.import_module('int_hash_lib.utils')
    func_name = mapping.get(k, None)
    hs_func = getattr(mod, func_name) if func_name else None
    nv = v
    if hs_func is None:
        return
    if isinstance(v, (typing.List, typing.Tuple)):
        nv = [hs_func(str_or_None(i), salt=salt, region=region) for i in v]
    else:
        nv = hs_func(str_or_None(v), salt=salt, region=region)
    return nv


def make_hash_from_dict(data: typing.Dict, salt: str, region: str,
                        first_only_keys: (typing.List, typing.Tuple), mapping: typing.Dict,
                        split_comma: (typing.List, typing.Tuple)) -> typing.Dict:
    item = {}

    def get_first(data):
        if isinstance(data, typing.Dict):
            return {'agg0': data.get('agg0')} if data.get('agg0') is not None else {}
        if isinstance(data, (typing.List, typing.Tuple)):
            data = [{'agg0': d.get('agg0')} for d in data if d.get('agg0') is not None]
        return data

    def one_or_many(item):
        if len(item.values()) == 1:
            return list(item.values())[0]
        return item

    for k, v in data.items():
        hs = prepare_hash_by_key(k, v, salt=salt, region=region, mapping=mapping)
        logger.debug('key: %s, value: %s, hs_result: %s', (k, v, hs))
        if hs:
            if k in first_only_keys:
                hs = get_first(hs)
            if isinstance(hs, (typing.List, typing.Tuple)):
                val = ','.join([str(one_or_many(d)) for d in hs])
                item[k] = val
            else:  # here is dict
                val = one_or_many(hs)
                if isinstance(val, typing.Dict):
                    for ik, iv in val.items():
                        item['%s_%s' % (k, ik)] = iv
                else:
                    item[k] = val
        else:
            if isinstance(v, (datetime.datetime, datetime.date)):
                v = str(v)
            elif isinstance(v, (typing.List, typing.Tuple)):
                v = ','.join([str(d) for d in v])
            item[k] = v
    for k, v in item.items():
        if not v and v != 0:
            item[k] = None
    return item


def make_hash_from_csv(in_file, salt: str, region: str, first_only_keys: (typing.List, typing.Tuple),
                       mapping: typing.Dict, split_comma: (typing.List, typing.Tuple), schema=None):
    # schema - it is a marshmallow Schema object type
    try:
        in_file = in_file.decode('utf-8').splitlines()
    except AttributeError:
        pass
    rd = csv.reader(in_file, delimiter=',', quotechar='"')
    headers = next(rd, None)
    with StringIO() as f:
        for idx, row in enumerate(rd):
            # row = [str(r).strip() if len(r.split(',')) < 2 else list(filter(None, r.split(','))) for r in row]
            row = [str(r).strip() for r in row]
            row_dict = dict(zip(headers, row))
            for k, v in row_dict.items():
                if k in split_comma:
                    row_dict[k] = [r.strip() for r in v.split(',') if r]
            if schema is not None:
                invalid_fields = schema().validate(data=row_dict).keys()
                for i in invalid_fields:
                    row_dict[i] = None
                row_dict = schema().load(data=row_dict, unknown='include')
            data = make_hash_from_dict(data=row_dict, salt=salt, region=region, first_only_keys=first_only_keys,
                                       mapping=mapping, split_comma=split_comma)
            if idx == 0:
                csv.writer(f).writerow(list(data.keys()))
            csv.writer(f).writerow(list(data.values()))
        return f.getvalue()



from setuptools import setup

setup(
    name='int-hash-lib',
    version='0.0.12',
    packages=['int_hash_lib', 'int_hash_lib.utils'],
    url='https://bitbucket.org/a_vento/int-hash-lib/src/master/',
    license='',
    author='MagicScore',
    author_email='',
    description='lib to hash data to int',
    install_requires=['phonenumbers<9']
)



import logging
import typing

import phonenumbers
from int_hash_lib import make_hs

logger = logging.Logger(__name__)


def build_phone_agg0(raw_data: str, region: str, salt: str) -> typing.Dict:
    result = {'raw_data': raw_data, 'data': None, 'hs': None, 'error': None}
    try:
        data = phonenumbers.parse(raw_data, region=region)
        if sum((phonenumbers.is_valid_number(data), phonenumbers.is_possible_number(data),
                region == phonenumbers.region_code_for_number(data))) != 3:
            error = 'data: %s, error: invalid phone number' % raw_data
            logger.error(error)
            result.update({'error': error})
            return result
        result.update({'data': str(data.national_number), 'hs': make_hs(str(data.national_number), salt)})
    except phonenumbers.phonenumberutil.NumberParseException as e:
        error = 'data: %s, error: %s' % (raw_data, e)
        result.update({'error': error})
        logger.error(error)
    return result


def build_phone_agg_remove_last_symbol(raw_data: str, salt, symbols_count: int) -> typing.Dict:
    result = {'raw_data': raw_data, 'data': None, 'hs': None, 'error': None}
    try:
        data = raw_data[:-symbols_count]
        hs = make_hs(data, salt)
        result.update({'raw_data': raw_data, 'data': data, 'hs': hs, 'error': None})
    except (AttributeError, ValueError, TypeError, IndexError) as e:
        error = 'build_phone_agg_remove_last_symbol error for value: %s, error: %s' % (raw_data, e)
        logger.error(error)
        result.update({'error': error})
    return result


def collect_phone_aggregates(raw_data: str, salt: str, region: str, **kwargs) -> typing.Dict:
    agg0 = build_phone_agg0(raw_data, region, salt)
    agg1 = build_phone_agg_remove_last_symbol(agg0['data'], salt, 1)
    agg2 = build_phone_agg_remove_last_symbol(agg0['data'], salt, 2)
    agg3 = build_phone_agg_remove_last_symbol(agg0['data'], salt, 3)
    agg4 = build_phone_agg_remove_last_symbol(agg0['data'], salt, 4)
    return {
        'agg0': agg0['hs'],
        'agg1': agg1['hs'],
        'agg2': agg2['hs'],
        'agg3': agg3['hs'],
        'agg4': agg4['hs'],
    }


__all__ = ['collect_phone_aggregates']
